import java.util.Date;
import java.util.Hashtable;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.InitialContext;

public class Receive
{


  public static void main(String[] args)
    throws Exception
  {
    new Receive().run(args);
  }


  private Hashtable getEnv(String host)
  {
    Hashtable env = new Hashtable();
    env.put("java.naming.factory.initial",
            "org.jnp.interfaces.NamingContextFactory");
    env.put("java.naming.provider.url", "jnp://" + host + ":1099");
    return env;
  }


  public void run(String[] args)
    throws Exception
  {
    InitialContext ctx = new InitialContext(getEnv(args[0]));
    ConnectionFactory cf = (ConnectionFactory) ctx.lookup("/ConnectionFactory");
    Queue q = (Queue) ctx.lookup("queue/ExampleQueue");
    Connection conn = null;
    Session s = null;
    MessageConsumer consumer = null;
    try {
      conn = cf.createConnection();
      s = conn.createSession(true, Session.SESSION_TRANSACTED);
      consumer = s.createConsumer(q);
      consumer.setMessageListener(new ReceiveMessageListener(s));
      conn.start();
      while (true) {
        Thread.currentThread().sleep(100);
      }
    } finally {
      if (consumer != null) {
        consumer.close();
      }
      if (s != null) {
        s.close();
      }
      if (conn != null) {
        conn.close();
      }
    }
  }


  public class ReceiveMessageListener implements MessageListener
  {
    Session session;

    public ReceiveMessageListener(Session s)
    {
      this.session = s;
    }


    public void onMessage(final Message msg)
    {
      long l = System.currentTimeMillis();
      try {
        TextMessage m = (TextMessage) msg;
        System.out.println(
          String.format(
            "[%1$tH:%1$tM:%1$tS:%1$tL] received message: %2$s",
            new Date(l),
            m.getText()));
        this.session.rollback();
      } catch (JMSException e) {
        e.printStackTrace();
      }
    }
  }
}
